.w3tc-widget-ps-rules {
    margin: 1em 0;
    background: #fff url(../img/ps_bar.gif) repeat-y;
}

.w3tc-widget-ps-rules a {
    color: #00f;
    text-decoration: none;
}

.w3tc-widget-ps-rules li,
.w3tc-widget-ps-rules p {
    margin: 0;
}

.w3tc-widget-ps-rule {
    line-height: 20px;
    clear: left;
}

.w3tc-widget-ps-rule p {
    margin-left: 40px;
    line-height: 20px;
}

.w3tc-widget-ps-icon {
    width: 31px;
    height: 20px;
    position: relative;
    float: left;
}

.w3tc-widget-ps-icon div {
    display: block;
    background: url(../img/ps_scores.png);
    width: 14px;
    height: 14px;
    position: relative;
    top: 3px;
    left: 8px;
}

.w3tc-widget-ps-priority-high .w3tc-widget-ps-icon div {
    background-position: 0;
}

.w3tc-widget-ps-priority-medium .w3tc-widget-ps-icon div {
    background-position: -14px;
}

.w3tc-widget-ps-priority-low .w3tc-widget-ps-icon div {
    background-position: -28px;
}

.w3tc-widget-ps-expand {
    float: left;
    width: 30px;
    height: 20px;
    text-align: center;
}

.w3tc-widget-ps-expand a {
    color: #66d;
}

.w3tc-manual-link {
    background-color: #efefef;
}
.w3tc-manual-link p {
    margin: 10px;
    padding:0;
    font-size:12px;
}
#w3tc-dashboard-widgets ul {
    margin: 0;
    padding: 0;
}
#w3tc-dashboard-widgets li {
    list-style-type: none;
    margin-bottom: 15px;
}

#w3tc-dashboard-widgets #normal-sortables #w3tc_spreadtheword label{
    display: block;
    width: 70px;
    float: left;
    line-height: 30px;
}

#w3tc-dashboard-widgets h3.hndle img {
    vertical-align: middle;
    margin-right:10px;
}

#w3tc_services label {
    font-weight: bold;
    margin-left: 10px;
    display: inline-block;
    width: 270px;
}

#w3tc_services label span {
    font-weight: normal;
}

#w3tc_services  input.w3tc-service {
    float: left;
    line-height: 40px;
}
#w3tc_spreadtheword .inside {
    margin-right: 30px;
}

#w3tc_services p {
    margin-left: 30px;
}

#w3tc-dashboard-widgets #normal-sortables .postbox {
    width:360px;
}

#w3tc-dashboard-widgets #side-sortables .postbox {
    border:none;
    box-shadow:none;
    -webkit-box-shadow: none;
    -webkit-border-radius: 0;
    border-radius: 0;
}

#postbox-container-right {
    width: 196px;
    white-space: nowrap;
    display: inline-block;
    float: right;
    margin-left: -200px;
}
#w3tc-dashboard-widgets #postbox-container-3 {
    width: 100%;white-space: normal;
}
.w3tc-enabled {
    color: #090;
    font-weight: 700
}

.w3tc-disabled {
    color: #f00;
    font-weight: 700
}

#normal-sortables  #w3tc_new_relic {
    width: 360px;
}
#new-relic-widget ul li, .maxcdn-netdna-widget-base ul li {
    margin-bottom: 3px;
    padding: 0;
}
#new-relic-widget ul li span, .maxcdn-netdna-widget-base  .summary ul li span{
    display: block;
    font-weight: bold;
    width: 110px;
    float:left;
}
#new-relic-top-list {
    margin-left: -10px;
    margin-right: -10px;
}
#new-relic-top-list h4 {
    padding-left: 10px;
}
#new-relic-widget .slowest th{
    text-align: left;
}
#new-relic-widget #new-relic-summary{
    display: block;
    width: 175px;
    float:left;
}
#new-relic-widget h4 {
    font-size:13px;
    margin: 0 0 10px;
}
#new-relic-widget h5 {
    font-size:12px;
    font-weight: bold;
    margin-top: 10px;
    margin-left: 0;
    margin-bottom: 0;
    border-top: 1px solid #d2d2d2;
    border-bottom: 1px solid #d2d2d2;
    cursor: pointer;
    padding: 5px 10px;

}

#new-relic-widget h5 .handlediv {
    display: block;
    background: url(../img/open.png) 0 2px no-repeat;
    width: 14px;
    height: 14px;
    position: relative;
}

#new-relic-widget h5 .handlediv.close {
    display: block;
    background: url(../img/close.png) 0 6px no-repeat;
    width: 14px;
    height: 14px;
    position: relative;
}

#new-relic-extra-metrics {
    float:left;
}
#new-relic-top-list {
    clear:both;
}
#new-relic-top-list h4 {
    padding-top: 20px;
    color: #8F8F8F;
}
#new-relic-top-list .slowest td, #new-relic-top-list p {
    font-family: Arial, Verdana, sans-serif;
    border:none;
    padding: 0 0 0 10px;
}
.maxcdn-netdna-widget-base h4 {
    text-align: center;
}
.maxcdn-netdna-widget-base .summary h4 {
    color: #8F8F8F;
    text-align: left;
}
.maxcdn-netdna-widget-base .status,.maxcdn-netdna-widget-base .summary {
    border-bottom: 1px solid #d2d2d2;
}

.maxcdn-netdna-widget-base .wrapper {
    margin-left: -10px;
    margin-right: -10px;
}
.maxcdn-netdna-widget-base  .area {
    padding-left: 10px;
    padding-right: 10px;
}


.maxcdn-netdna-widget-base .tools {
    margin-top:15px;
}
.maxcdn-netdna-widget-base .tools li {
    display:inline-block;
    margin-right:10px;
}
.maxcdn-netdna-widget-base .summary li.large span {
    width: 210px;
}
.maxcdn-netdna-widget-base  ul.file_hits li {
    line-height: 12px;
    font-size:10px;
}
.maxcdn-netdna-widget-base  ul.file_hits li span{
    color: #fff;
    padding-top: 2px;
    padding-left: 5px;
}
.maxcdn-netdna-widget-base .status p,  .maxcdn-netdna-widget-base .charts p{
    color:#8F8F8F
}
.maxcdn-netdna-widget-base .account_status  {
    font-weight: bold;
    color: #000;
}
.maxcdn-netdna-widget-base .content-zone {
    color: #c0d9e4
}
.maxcdn-netdna-widget-base.sign-up h4 {
    text-align: left;
    margin-bottom: 0;
    padding-bottom: 0;
}
.maxcdn-netdna-widget-base.sign-up p{
    margin-top: 4px;
    padding-top: 0;

}
.maxcdn-netdna-widget-base.sign-up p span.desc {
    color:#8F8F8F;
}
