<?php

/**
 * CSS and JS mime types
 */
return array(
    'css' => 'text/css', 
    'htc' => 'text/x-component',
    'less' => 'text/css',
    //JS - varieties
    'js' => 'application/x-javascript',
    'js2' => 'application/javascript',
    'js3' => 'text/javascript',
    'js4' => 'text/x-js',
);
