
html, body, div, span, applet, object, iframe,
h1, h2, h3, h4, h5, h6, p, blockquote, pre,
a, abbr, acronym, address, big, cite, code,
del, dfn, em, img, ins, kbd, q, s, samp,
small, strike, sub, sup, tt, var,
center,
dl, dt, dd, ol, ul, li,
fieldset, form, label, legend,
table, caption, tbody, tfoot, thead, tr, th, td,
article, aside, canvas, details, embed,
figure, figcaption, footer, header, hgroup,
menu, nav, output, ruby, section, summary,
time, mark, audio, video {
  margin: 0;
  padding: 0;
  border: 0;
  font-size: 100%;
  font: inherit;
  vertical-align: baseline;
}

/* HTML5 display-role reset for older browsers */
article, aside, details, figcaption, figure,
footer, header, hgroup, menu, nav, section {
  display: block;
}

ol, ul {
  list-style: none;
}

blockquote, q {
  quotes: none;
}

blockquote:before, blockquote:after,
q:before, q:after {
  content: '';
  content: none;
}

table {
  border-collapse: collapse;
  border-spacing: 0;
}

/******************************/
/** MY RESETS **/
* {
  margin: 0;
  outline: none;
  box-sizing: border-box;
}

html, body {
  height: 100%;
}

input {
  padding: 0;
  background: #ccc;
  border: none;
  appearance: normal;
  -moz-appearance: normal;
  -webkit-appearance: normal;
}

textarea {
  appearance: normal;
  background: #ccc;
  border: none;
  padding: 0;
  resize: none;
  height: auto;
}

ul li {
  list-style-type: none;
}

a {
  text-decoration: none;
}

/** CUSTOM SYLES **/
.clear {
  clear: both;
}

.con {
  margin: 0 auto;
  position: relative;
}

/*
.foo {
  @include placeholder {
    color: green;
  }
}
*/
.con, .top, .top .menu, .slider li, .slider .txt2, .bttn2.orange-bg, footer .con {
  *zoom: 1;
}
.con:before, .top:before, .top .menu:before, .slider li:before, .slider .txt2:before, .bttn2.orange-bg:before, .con:after, .top:after, .top .menu:after, .slider li:after, .slider .txt2:after, .bttn2.orange-bg:after {
  content: " ";
  display: table;
}
.con:after, .top:after, .top .menu:after, .slider li:after, .slider .txt2:after, .bttn2.orange-bg:after {
  clear: both;
}

/*
*http://www.basic-slider.com/
*https://github.com/jcobb/basic-jquery-slider/
*/
ul.bjqs {
  position: relative;
  list-style: none;
  padding: 0;
  margin: 0;
  overflow: hidden;
}
ul.bjqs li.bjqs-slide {
  position: absolute;
}

ul.bjqs-controls {
  list-style: none;
  margin: 0;
  padding: 0;
  z-index: 9999;
}
ul.bjqs-controls.v-centered li.bjqs-next a {
  right: 0;
}
ul.bjqs-controls.v-centered li.bjqs-prev a {
  left: 0;
}
ul.bjqs-controls.v-centered li a {
  position: absolute;
}

ol.bjqs-markers {
  list-style: none;
  padding: 0;
  margin: 0;
  width: 100%;
}

ol.bjqs-markers.h-centered {
  text-align: center;
}

ol.bjqs-markers li {
  display: inline;
}

ol.bjqs-markers li a {
  display: inline-block;
}

p.bjqs-caption {
  display: block;
  width: 96%;
  margin: 0;
  padding: 2%;
  position: absolute;
  bottom: 0;
}

pre {
  font-family: "Source Code Pro", monospace;
  display: block;
  padding: 9.5px;
  margin: 0 0 10px;
  font-size: 13px;
  line-height: 20px;
  word-break: break-all;
  word-wrap: break-word;
  white-space: pre;
  white-space: pre-wrap;
  background-color: whiteSmoke;
  border: 1px solid #CCC;
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
}

#container {
  max-width: 620px;
  margin: 0 auto;
  padding-bottom: 80px;
}

#banner-fade,
#banner-slide {
  margin-bottom: 60px;
}

ul.bjqs-controls.v-centered li a {
  display: block;
  padding: 10px;
  background: #fff;
  color: #000;
  text-decoration: none;
}

ul.bjqs-controls.v-centered li a:hover {
  background: #000;
  color: #fff;
}

ol.bjqs-markers li a {
  padding: 5px 10px;
  background: #000;
  color: #fff;
  margin: 5px;
  text-decoration: none;
}

ol.bjqs-markers li.active-marker a,
ol.bjqs-markers li a:hover {
  background: #999;
}

p.bjqs-caption {
  background: rgba(255, 255, 255, 0.5);
}

form {
  margin-top: 20px;
}
form span {
  color: #707173;
  font-size: 12px;
  margin: 7px 0 14px 20px;
  display: block;
}
@media screen and (max-width: 500px) {
  form br {
    display: none;
  }
}
form .text-input {
  width: 49%;
  margin: 5px 6px 5px 0;
  padding: 0 16px;
  height: 40px;
  line-height: 40px;
  font-size: 15px;
  color: #707173;
  background-color: #f4f5f7;
  font-family: Arial;
}
@media screen and (max-width: 1200px) {
  form .text-input {
    width: 49%;
    margin: 5px 1% 5px 0;
  }
  .articles:nth-of-type(2n + 1) {
	margin-left: 0.7% !important;
  }
}
@media screen and (max-width: 640px) {
  form .text-input {
    width: 100%;
    margin: 5px 0 5px 0;
  }
}
form .text-input::-webkit-input-placeholder {
  font-family: Arial;
  color: #707173;
}
form .text-input:-moz-placeholder {
  font-family: Arial;
  color: #707173;
}
form .text-input::-moz-placeholder {
  font-family: Arial;
  color: #707173;
}
form .text-input:-ms-input-placeholder {
  font-family: Arial;
  color: #707173;
}
form .text-input:nth-of-type(2n + 2) {
  margin-right: 0;
}
form .textarea {
  font-family: Arial;
  width: 100%;
  margin: 5px 6px 5px 0;
  padding: 16px 16px;
  font-size: 15px;
  color: #707173;
  background-color: #f4f5f7;
  box-sizing: border-box;
  min-height: 140px;
}
form .textarea::-webkit-input-placeholder {
  font-family: Arial;
  color: #707173;
}
form .textarea:-moz-placeholder {
  font-family: Arial;
  color: #707173;
}
form .textarea::-moz-placeholder {
  font-family: Arial;
  color: #707173;
}
form .textarea:-ms-input-placeholder {
  font-family: Arial;
  color: #707173;
}
form label {
  font-size: 12px;
  color: #939498;
  margin-left: 15px;
}
@media screen and (max-width: 500px) {
  form label {
    display: none;
  }
}

@font-face {
    font-family: 'Dax';
    src: url('fonts2/DaxProLight.eot');
    src: url('fonts2/DaxProLight.eot') format('embedded-opentype'),
         url('fonts2/DaxProLight.woff2') format('woff2'),
         url('fonts2/DaxProLight.woff') format('woff'),
         url('fonts2/DaxProLight.ttf') format('truetype'),
         url('fonts2/DaxProLight.svg#DaxProLight') format('svg');
}
body {
  font-family: Arial;
}

.wrapper {
  width: 100%;
  min-height: 100%;
}

.con {
  width: 1200px;
}
@media screen and (max-width: 1200px) {
  .con {
    width: 100% !important;
    box-sizing: border-box;
    padding: 0 20px;
  }
}

@media screen and (max-width: 1200px) {
  .bjqs-wrapper {
    width: 100% !important;
    height: 205px !important;
    display: none;
  }
}
@media screen and (max-width: 768px) {
  .bjqs-wrapper {
    height: 370px !important;
  }
}

@media screen and (max-width: 768px) {
  .bjqs {
    height: 370px !important;
  }
}

.resp {
  display: none;
}
@media screen and (max-width: 1200px) {
  .resp {
    display: block;
  }
}

@media screen and (max-width: 1200px) {
  .no-resp {
    display: none;
  }
}

.top {
  padding-top: 30px;
}
.top .logo {
  float: left;
}
@media screen and (max-width: 1200px) {
  .top .logo {
    width: 20%;
  }
}
@media screen and (max-width: 768px) {
  .top .logo {
    width: auto;
    margin-bottom: 20px;
  }
}
.top .social {
  float: right;
  text-align: right;
  margin-bottom: 6px;
  width: 50%;
}
@media screen and (max-width: 768px) {
  .top .social {
    float: none;
    text-align: center;
    width: 100%;
  }
}
.top .social li {
  display: inline-block;
  margin-left: 8px;
  -webkit-transition: opacity 0.5s ease-in-out;
  -moz-transition: opacity 0.5s ease-in-out;
  -ms-transition: opacity 0.5s ease-in-out;
  -o-transition: opacity 0.5s ease-in-out;
  transition: opacity 0.5s ease-in-out;
}
.top .social li:first-child {
  margin-left: 0;
}
.top .social li:hover {
  /* For IE 5,6,7,8,9 */
  zoom: 1;
  -ms-filter: "progid:DXImageTransform.Microsoft.Alpha(Opacity=70)";
  filter: progid:DXImageTransform.Microsoft.Alpha(opacity=$ie_opacity);
  filter: alpha(opacity=70);
  /* Older than Firefox 0.9 */
  -moz-opacity: 0.7;
  /* Safari 1.x (pre WebKit!) */
  -khtml-opacity: 0.7;
  /* Modern */
  opacity: 0.7;
}
.top .lang {
  float: right;
  text-align: right;
  width: 50%;
  margin-bottom: 7px;
}
@media screen and (max-width: 768px) {
  .top .lang {
    float: none;
    width: 100%;
    text-align: center;
    margin: 15px 0;
  }
}
.top .lang li {
  display: inline-block;
  margin-left: 11px;
}
.top .lang li.active a {
  font-weight: 700;
}
.top .lang li:first-child {
  margin-left: 0;
}
.top .lang li a {
  font-size: 12px;
  color: #3e3f40;
  -webkit-transition: all 0.5s ease-in-out;
  -moz-transition: all 0.5s ease-in-out;
  -ms-transition: all 0.5s ease-in-out;
  -o-transition: all 0.5s ease-in-out;
  transition: all 0.5s ease-in-out;
}
@media screen and (max-width: 768px) {
  .top .lang li a {
    font-size: 16px;
  }
}
.top .lang li:hover a {
  font-weight: 700;
  color: #37c0ee;
  text-decoration: underline;
}
.top form {
  float: right;
  text-align: right;
  width: 50%;
  margin-top: 0 !important;
}
@media screen and (max-width: 768px) {
  .top form {
    float: none;
    width: 100%;
  }
}
.top form input[type="search"]::-webkit-search-cancel-button {
  display: none;
}
.top form input {
  display: inline-block;
  height: 40px;
  vertical-align: top;
}
.top form input[type="search"] {
  line-height: 40px;
  width: 257px;
  padding: 0 16px;
  font-size: 15px;
  color: #707173;
  background-color: #e9eaec;
}
@media screen and (max-width: 768px) {
  .top form input[type="search"] {
    width: expression(100% - 41px);
    width: -webkit-calc(100% - 41px);
    width: -moz-calc(100% - 41px);
    width: -webkit-calc(100% - 41px);
    width: calc(100% - 41px);
  }
}
.top form input[type="search"]::-webkit-input-placeholder {
  font-family: Arial;
  opacity: 1;
  font-size: 15px;
  color: #707173;
}
.top form input[type="search"]:-moz-placeholder {
  font-family: Arial;
  opacity: 1;
  font-size: 15px;
  color: #707173;
}
.top form input[type="search"]::-moz-placeholder {
  font-family: Arial;
  opacity: 1;
  font-size: 15px;
  color: #707173;
}
.top form input[type="search"]:-ms-input-placeholder {
  font-family: Arial;
  opacity: 1;
  font-size: 15px;
  color: #707173;
}
.top form input[type="submit"] {
  width: 40px;
  background-color: #59595c;
  margin-left: -4px;
  background-image: url(img/search.png);
  background-position: center;
  background-repeat: no-repeat;
  cursor: pointer;
  -webkit-transition: all 0.5s ease-in-out;
  -moz-transition: all 0.5s ease-in-out;
  -ms-transition: all 0.5s ease-in-out;
  -o-transition: all 0.5s ease-in-out;
  transition: all 0.5s ease-in-out;
}
@media screen and (max-width: 768px) {
  .top form input[type="submit"] {
    width: 40px;
  }
}
.top form input[type="submit"]:hover {
  background-color: #1a76bc;
}
.top .max-h {
  max-height: 100%;
}
.top .menu {
  float: right;
  text-align: right;
}
@media screen and (max-width: 768px) {
  .top .menu {
    float: none;
    width: 100%;
    text-align: left;
    margin-top: 15px;
  }
}
.top .menu li {
  float: left;
  background-color: #1a76bc;
  -webkit-transition: all 0.5s ease-in-out;
  -moz-transition: all 0.5s ease-in-out;
  -ms-transition: all 0.5s ease-in-out;
  -o-transition: all 0.5s ease-in-out;
  transition: all 0.5s ease-in-out;
}
@media screen and (max-width: 768px) {
  .top .menu li {
    float: none;
    width: 100%;
    border-bottom: 1px solid #fff;
  }
}
.top .menu li:hover, .top .menu li.active {
  background-color: #292561;
}
@media screen and (max-width: 768px) {
  .top .menu li.drop {
    background-image: url(img/down.png);
    background-repeat: no-repeat;
    background-position: right 10px top 15px;
  }
}
.top .menu li.drop ul {
  position: absolute;
  z-index: 2;
  max-height: 0px;
  overflow: hidden;
  border-left: 1px solid #292561;
  border-right: 1px solid #292561;
  -webkit-transition: all 0.5s ease-in-out;
  -moz-transition: all 0.5s ease-in-out;
  -ms-transition: all 0.5s ease-in-out;
  -o-transition: all 0.5s ease-in-out;
  transition: all 0.5s ease-in-out;
}
@media screen and (max-width: 768px) {
  .top .menu li.drop ul {
    position: relative;
    border: none;
    display: none;
    transition: none;
    max-height: inherit;
  }
}
.top .menu li.drop ul li {
  display: block;
  float: none;
  background-color: #292561;
  min-width: 220px;
  border-bottom: 1px solid #292561;
  -webkit-transition: all 0.5s ease-in-out;
  -moz-transition: all 0.5s ease-in-out;
  -ms-transition: all 0.5s ease-in-out;
  -o-transition: all 0.5s ease-in-out;
  transition: all 0.5s ease-in-out;
}
@media screen and (max-width: 768px) {
  .top .menu li.drop ul li {
    border-top: 1px solid #fff;
    border-bottom: none;
    background-color: #1a76bc;
    transition: none;
  }
}
.top .menu li.drop ul li:hover {
  background-color: #1a76bc;
}
@media screen and (max-width: 768px) {
  .top .menu li.drop ul li:hover {
    background-color: #1a76bc;
  }
}
.top .menu li.drop ul li a {
  text-align: left;
}
@media screen and (max-width: 768px) {
  .top .menu li.drop ul li a {
    text-align: left;
    padding: 0 34px;
  }
}
.top .menu li.drop:hover ul {
  display: block;
  max-height: 500px;
}
@media screen and (max-width: 768px) {
  .top .menu li.drop:hover ul {
    display: none;
    max-height: inherit;
  }
}
.top .menu li a {
  display: block;
  padding: 0 33px;
  height: 40px;
  line-height: 40px;
  color: white;
  font-size: 15px;
  text-align: center;
}
@media screen and (max-width: 768px) {
  .top .menu li a {
    text-align: left;
    padding: 0 17px;
	margin-right: 30px;
  }
}

.slider {
  margin-bottom: 9px;
}
@media screen and (max-width: 1200px) {
  .slider {
    min-height: 205px;
    height: auto !important;
  }
}
.slider li {
  width: 100%;
  background-size: cover;
}
.slider .txt1 {
  float: left;
  padding: 76px 0px;
  width: 200px;
  max-height: 200px;
  overflow: hidden;
  color: #fff;
  font-size: 26px;
  font-family: Dax;
  padding-left: 22px;
  box-sizing: border-box;
  background-color: rgba(10, 78, 132, 0.7);
  margin-left: 9px;
  margin-top: 87px;
}
.slider .txt1:nth-child(1) {
  margin-left: 192px;
}
@media screen and (max-width: 1200px) {
  .slider .txt1 {
    display: none;
  }
}
.slider .txt2 {
  float: left;
  width: 410px;
  max-height: 200px;
  min-height: 200px;
  overflow: hidden;
  color: #fff;
  font-size: 18px;
  font-family: Dax;
  padding-left: 22px;
  box-sizing: border-box;
  background-color: rgba(10, 78, 132, 0.7);
  margin-left: 9px;
  margin-top: 87px;
  line-height: 26px;
  padding-top: 28px;
  padding-right: 28px;
}
.slider .txt2 a {
  display: block;
  font-size: 16px;
  text-transform: uppercase;
  color: #26a8e0;
  -webkit-transition: all 0.3s ease-in-out;
  -moz-transition: all 0.3s ease-in-out;
  -ms-transition: all 0.3s ease-in-out;
  -o-transition: all 0.3s ease-in-out;
  transition: all 0.3s ease-in-out;
}
.slider .txt2 a:hover {
  color: #fff;
  text-decoration: underline;
}
@media screen and (max-width: 1200px) {
  .slider .txt2 {
    margin-top: 0;
    margin-left: 0;
    width: 70%;
    font-size: 18px;
    padding-top: 15px;
    line-height: 26px;
    max-height: inherit;
    min-height: 137px;
    padding-bottom: 15px;
  }
}
@media screen and (max-width: 375px) {
  .slider .txt2 {
    width: 100%;
    font-size: 16px;
  }
}

.ham {
  display: none;
}
@media screen and (max-width: 768px) {
  .ham {
    width: 35px;
    height: 32px;
    background: url(img/ham.png) no-repeat center;
    position: absolute;
    display: block;
    top: 35px;
    right: 35px;
    border: none;
  }
}

@media screen and (max-width: 768px) {
  .resp-drop {
    display: none;
    clear: both;
    border-top: 1px solid #ccc;
    padding-top: 15px;
    padding-bottom: 15px;
    border-bottom: 1px solid #ccc;
    margin-bottom: 15px;
  }
}

.bttn2 {
  width: 200px;
  display: inline-block;
  max-height: 68px;
  height: 68px;
  padding: 22px 20px;
  box-sizing: border-box;
  font-family: Dax;
  color: #fff;
  text-transform: uppercase;
  text-align: center;
  vertical-align: top;
  margin-top: 10px;
  margin-right: 6px;
  font-size: 19px;
  -webkit-transition: all 0.5s ease-in-out;
  -moz-transition: all 0.5s ease-in-out;
  -ms-transition: all 0.5s ease-in-out;
  -o-transition: all 0.5s ease-in-out;
  transition: all 0.5s ease-in-out;
}
@media screen and (max-width: 1200px) {
  .bttn2 {
    width: 33.33%;
    margin-right: 0;
    float: left;
    display: block;
    position: relative;
    top: 0px;
    margin-top: 0px;
  }
}
@media screen and (max-width: 960px) {
  .bttn2 {
    font-size: 17px;
  }
}
@media screen and (max-width: 768px) {
  .bttn2 {
    font-size: 20px;
    width: 70%;
    margin-top: -1px;
    top: 1px;
    padding: 21px 28px;
    height: 61px;
    max-height: 61px;
  }
}
@media screen and (max-width: 375px) {
  .bttn2 {
    width: 100%;
  }
}
.bttn2.blue-bg {
  margin-left: 192px;
  background-color: #26a8e0;
}
@media screen and (max-width: 1200px) {
  .bttn2.blue-bg {
    margin-left: 0;
  }
}
.bttn2.vio-bg {
  background-color: #a21f60;
  padding: 11px 31px;
}
@media screen and (max-width: 1200px) {
  .bttn2.vio-bg {
    padding: 25px 28px;
  }
}
@media screen and (max-width: 768px) {
  .bttn2.vio-bg {
    padding: 20px 28px;
  }
}
.bttn2.oneline {
  background-color: #a21f60;
  padding: 21px 28px;
}
.bttn2.orange-bg {
  background-color: #f15a29;
  padding: 11px 31px;
}
.bttn2:hover {
background:#292561;
}

.info-block {
  width: 589px;
  min-height: 280px;
  display: inline-block;
  background-repeat: no-repeat;
  background-position: center;
  background-size: cover;
  margin: 9px 8px;
}
@media screen and (max-width: 1200px) {
  .info-block {
    display: block;
    float: left;
    margin: 0;
    margin-left: 0.7%;
    width: 49.6%;
  }
}
@media screen and (max-width: 768px) {
  .info-block {
    width: 100%;
    margin: 15px 0;
  }
}
.info-block:nth-of-type(2n + 1) {
  margin-left: 0;
}
.info-block:nth-of-type(2n + 2) {
  margin-right: 0;
}
.info-block span {
  height: 58px;
  line-height: 58px;
  padding: 0 71px 0 22px;
  display: inline-block;
  font-family: Dax;
  text-transform: uppercase;
  color: #fff;
  font-size: 20px;
}
@media screen and (max-width: 375px) {
  .info-block span {
    width: 100%;
    padding-right: 0;
  }
}
.info-block span.orange-bg {
  background-color: rgba(240, 110, 62, 0.9);
}

.articles {
  display: inline-block;
  width: 588px;
  margin-top: 9px;
  margin-bottom: 25px;
  vertical-align: top;
  margin-left: 9px;
  margin-right: 9px;
}
.articles:nth-of-type(2n + 1) {
  margin-left: 0;
}
.articles:nth-of-type(2n + 2) {
  margin-right: 0;
}
@media screen and (max-width: 1200px) {
  .articles {
    margin: 0;
    margin-left: 0.7%;
    display: block;
    float: left;
    width: 49.6%;
  }
}
@media screen and (max-width: 768px) {
  .articles {
    width: 100%;
    margin: 0;
  }
}
.articles h2 {
  font-family: Dax;
  font-size: 20px;
  color: #fff;
  text-transform: uppercase;
  width: 100%;
  height: 60px;
  line-height: 60px;
  padding: 0 23px;
  margin: 5px 0;
}
.articles h2.bg-blue {
  background-image: linear-gradient(#30bbe8, #38c1f0);
}
.articles h2.bg-green {
  background-color: #009347;
}
@media screen and (max-width: 768px) {
  .articles .news-item + .bttn1,
  .articles .article-item + .bttn1 {
    float: none;
    margin-left: auto;
    margin-right: auto;
    margin-bottom: 15px;
  }
}
.articles .news-item, .articles .article-item {
  margin: 7px 0;
  border: 1px solid #f8f9fa;
  -webkit-transition: all 0.5s ease-in-out;
  -moz-transition: all 0.5s ease-in-out;
  -ms-transition: all 0.5s ease-in-out;
  -o-transition: all 0.5s ease-in-out;
  transition: all 0.5s ease-in-out;
}
.articles .news-item:hover, .articles .article-item:hover {
  background-color: #1a76bc;
  border-color: #1a76bc;
}
.articles .news-item:hover h3, .articles .news-item:hover .date, .articles .news-item:hover p, .articles .article-item:hover h3, .articles .article-item:hover .date, .articles .article-item:hover p {
  color: #fff;
}
.articles .news-item a, .top-block a, .mid-con a {
color:#1a76bc;
}
.articles .news-item:hover a {
color:#fff;
}
.articles .news-item {
  padding: 27px 19px;
  min-height:123px;
}
.articles .article-item {
  padding: 25px 19px;
}
.articles .article-item h3 {
  margin-bottom: 0;
}
.articles h3 {
  font-weight: 700;
  font-size: 18px;
  color: #1a76bc;
  display: inline-block;
  vertical-align: top;
  width: 86%;
  margin-bottom: 5px;
  -webkit-transition: all 0.5s ease-in-out;
  -moz-transition: all 0.5s ease-in-out;
  -ms-transition: all 0.5s ease-in-out;
  -o-transition: all 0.5s ease-in-out;
  transition: all 0.5s ease-in-out;
}
@media screen and (max-width: 375px) {
  .articles h3 {
    width: 100%;
  }
}
.articles .date {
  display: inline-block;
  vertical-align: top;
  font-size: 12px;
  color: #b8b9bd;
  width: 13%;
  margin-top: 5px;
  -webkit-transition: all 0.5s ease-in-out;
  -moz-transition: all 0.5s ease-in-out;
  -ms-transition: all 0.5s ease-in-out;
  -o-transition: all 0.5s ease-in-out;
  transition: all 0.5s ease-in-out;
}
.articles p {
  margin-top: 5px;
  font-size: 14px;
  color: #8a8b8e;
  line-height: 18px;
  -webkit-transition: all 0.5s ease-in-out;
  -moz-transition: all 0.5s ease-in-out;
  -ms-transition: all 0.5s ease-in-out;
  -o-transition: all 0.5s ease-in-out;
  transition: all 0.5s ease-in-out;
}

.f-right {
  float: right;
}

.f-left {
  float: left;
}

.centered {
  margin: 0 auto;
}

.bttn1 {
  height: 40px;
  width:190px;
  line-height: 40px;
  padding: 0 37px;
  font-size: 15px;
  text-align:center;
  color: #fff;
  background-color: #1a76bc;
  display: table;
  border: none;
  cursor: pointer;
  -webkit-transition: all 0.3s ease-in-out;
  -moz-transition: all 0.3s ease-in-out;
  -ms-transition: all 0.3s ease-in-out;
  -o-transition: all 0.3s ease-in-out;
  transition: all 0.3s ease-in-out;
}
@media screen and (max-width: 500px) {
  .bttn1 {
    display: table;
    margin-left: auto;
    margin-right: auto;
    width: 100% !important;
  }
}
.bttn1:hover {
  background-color: #37c0ee;
}
.bttn1.wider {
  padding: 0 62px;
}

.upload {
  font-size: 15px;
  color: white;
  background-color: #59595c;
  display: inline-block;
  border: none;
  cursor: pointer;
  margin: 6px 0;
  height: 40px;
  line-height: 40px;
  background-image: url(img/upload.png);
  background-repeat: no-repeat;
  background-position: left 22px center;
  -webkit-transition: all 0.3s ease-in-out;
  -moz-transition: all 0.3s ease-in-out;
  -ms-transition: all 0.3s ease-in-out;
  -o-transition: all 0.3s ease-in-out;
  transition: all 0.3s ease-in-out;
}
@media screen and (max-width: 500px) {
  .upload {
    display: table;
    margin-left: auto;
    margin-right: auto;
    width: 100% !important;
  }
}
.upload:hover {
  background-color: #37c0ee;
}
.upload.wider {
  width: 190px;
}
.upload:nth-of-type(2) {
  margin-bottom: 10px;
}
@media screen and (max-width: 500px) {
  .upload:nth-of-type(2) {
    margin-bottom: 10px;
  }
}

footer {
  margin-top: 9px;
  padding: 30px 0px;
  width: 100%;
  background-color: #59595c;
}
footer .logo-sm {
  display: inline-block;
  vertical-align: top;
  margin-right: 40px;
}
footer ul {
  display: inline-block;
  min-width: 180px;
  margin-right: 10px;
  vertical-align: top;
}
@media screen and (max-width: 1200px) {
  footer ul {
    display: none;
  }
}
footer ul li {
  display: block;
  margin: 6px 0px;
}
footer ul li a {
  font-size: 15px;
  color: #b4b5b6;
  -webkit-transition: all 0.3s ease-in-out;
  -moz-transition: all 0.3s ease-in-out;
  -ms-transition: all 0.3s ease-in-out;
  -o-transition: all 0.3s ease-in-out;
  transition: all 0.3s ease-in-out;
}
footer ul li a:hover {
  font-weight: 700;
  color: #37c0ee;
  text-decoration: underline;
}
footer h4 {
  font-weight: 700;
  font-size: 18px;
  color: white;
  margin-bottom: 20px;
}
footer h4 a {
  color: #fff;
  -webkit-transition: all 0.3s ease-in-out;
  -moz-transition: all 0.3s ease-in-out;
  -ms-transition: all 0.3s ease-in-out;
  -o-transition: all 0.3s ease-in-out;
  transition: all 0.3s ease-in-out;
}
footer h4 a:hover {
  color: #1a76bc;
}
footer span {
  padding-top: 50px;
  font-size: 12px;
  color: #b4b5b6;
}
@media screen and (max-width: 1200px) {
  footer span {
    float: none !important;
    display: inline-block;
    vertical-align: middle;
    margin: 0 13px;
  }
}
@media screen and (max-width: 1060px) {
  footer span {
    float: none !important;
    display: inline-block;
    vertical-align: middle;
    margin: 0 13px;
	width:100%;
	padding-top:10px;
  }
}
@media screen and (max-width: 810px) {
  footer span {
    display: block;
    padding-top: 10px;
  }
}

#googleMap {
  width: 100%;
  height: 351px;
}

#googleMap + .mid-con {
  padding-left: 0;
}

.col {
  float: left;
  margin-top: 20px;
  box-sizing: border-box;
}
@media screen and (max-width: 1200px) {
  .col:first-of-type {
    display: none;
  }
}

.side-row {
  width: 24.17%;
}
@media screen and (max-width: 1200px) {
  .side-row {
    width: 100%;
  }
}

.middle-row {
  width: 50%;
  margin: 20px 9px;
}
@media screen and (max-width: 1200px) {
  .middle-row {
    width: 100%;
    margin: 0 0 5px 0;
  }
}
.middle-row .articles p {
  line-height: 24px;
}
.middle-row .mid-con {
  padding: 0 29px;
}
@media screen and (max-width: 375px) {
  .middle-row .mid-con {
    padding: 0;
  }
}
.middle-row .mid-con h3 {
  margin-top: 35px;
  font-size: 20px;
  color: #1a76bc;
  margin-bottom: 22px;
  font-family: Dax;
}
.middle-row .mid-con p {
  font-size: 15px;
  line-height: 24px;
  color: #59595c;
  margin: 16px 0;
}
.middle-row .mid-con ul li, .middle-row .mid-con ol li {
  font-size: 15px;
  line-height: 24px;
  color: #59595c;
}
.middle-row .mid-con a {
  color: #1a76bc;
  -webkit-transition: all 0.3s ease-in-out;
  -moz-transition: all 0.3s ease-in-out;
  -ms-transition: all 0.3s ease-in-out;
  -o-transition: all 0.3s ease-in-out;
  transition: all 0.3s ease-in-out;
}
.middle-row .mid-con a:hover {
  text-decoration: underline;
}

.top-block {
  background-color: #f4f5f7;
  min-height: 235px;
  padding: 30px 30px 30px 30px;
  box-sizing: border-box;
}
@media screen and (max-width: 1200px) {
  .top-block {
    min-height: inherit;
    padding: 30px 30px 30px 30px;
  }
}
.top-block h2 , .mid-con h2{
  font-family: Dax;
  font-size: 30px;
  color: #1a76bc;
  margin-bottom: 15px;
}
@media screen and (max-width: 375px) {
  .top-block h2 , .mid-con h2 {
    text-align: left;
  }
}
.top-block p {
  margin-top: 15px;
  font-size: 15px;
  color: #59595c;
  line-height: 24px;
  padding: 0;
}

.mid-con h3{
  font-family: Dax;
  font-size: 22px;
  color: #1a76bc;
  margin-bottom: 15px;
}

.round-block {
  background-repeat: no-repeat;
  background-position: center;
  background-size: cover;
  width: 290px;
  height: 235px;
  display: table;
  margin-bottom: 10px;
}
@media screen and (max-width: 1200px) {
  .round-block {
    display: inline-block;
    width: 31.5%;
    margin-left: -3px;
    margin-bottom: 15px;
    margin-right: 15px;
  }
}
@media screen and (max-width: 845px) {
  .round-block {
    width: 47%;
  }
}
@media screen and (max-width: 640px) {
  .round-block {
    width: 100%;
    margin-left: 0;
  }
}
.round-block span {
  font-family: Dax;
  color: white;
  text-transform: uppercase;
  display: table-cell;
}
@media screen and (max-width: 640px) {
.round-block span {
  width:100%;
  display:block !important;
}
}
.round-block span.middle {
  vertical-align: middle;
  padding-left: 34px;
  font-size: 30px;
}
.round-block span.top-text {
  padding: 21px 21px;
  font-size: 18px;
}

.side-menu {
  margin-top: 25px;
}
.side-menu ul li a {
  padding: 12px 32px;
  display: block;
  font-size: 15px;
  color: #1a76bc;
  -webkit-transition: all 0.3s ease-in-out;
  -moz-transition: all 0.3s ease-in-out;
  -ms-transition: all 0.3s ease-in-out;
  -o-transition: all 0.3s ease-in-out;
  transition: all 0.3s ease-in-out;
}
.side-menu ul li a.active, .side-menu ul li a:hover {
  color: white;
  font-weight: 700;
  background-color: #1a76bc;
}

.contact {
  min-height: 236px;
  width: 100%;
  background-repeat: no-repeat;
  background-position: center left;
  padding-left: 215px;
  box-sizing: border-box;
  margin: 10px 0;
  padding-top: 30px;
  background-color: #f4f5f7;
}

.conhover:hover {
  background-color: #e5e6e8;
  cursor:pointer;
}
@media screen and (max-width: 1200px) {
  .contact {
    width: 100%;
    margin: 15px 0;
  }
}
@media screen and (max-width: 500px) {
  .contact {
    width: 300px;
    padding-top: 250px;
    padding-left: 0;
    background-position: top 15px center;
    padding-bottom: 15px;
    display: table;
    margin: 15px auto;
    background-size: auto 230px;
  }
}
.contact h3 {
  font-weight: 700;
  margin-top: 0;
  font-size: 24px;
  margin-bottom: 0px;
  color: #1a76bc;
  font-family:Dax;
}
@media screen and (max-width: 500px) {
  .contact h3 {
    margin: 10px auto;
    text-align: center;
  }
}
.contact span {
  font-size: 15px;
  display: block;
}
@media screen and (max-width: 500px) {
  .contact span {
    display: table;
    margin: 0 auto;
  }
}
.contact .amats {
  color: #59595c;
}
.contact .location {
  color: #59595c;
  margin-top: 31px;
}
@media screen and (max-width: 500px) {
  .contact .location {
    margin-top: 15px;
  }
}
.contact .mail {
  color: #1a76bc;
  padding-left: 29px;
  background-image: url(img/mail.png);
  background-repeat: no-repeat;
  background-position: left center;
  margin-top: 42px;
}
@media screen and (max-width: 500px) {
  .contact .mail {
    margin-top: 12px;
  }
}
.contact .tel {
  padding-left: 29px;
  background-image: url(img/tel.png);
  background-repeat: no-repeat;
  background-position: left center;
  color: #797779;
  margin-top: 12px;
}

.col {
  float: left;
  margin-top: 20px;
  box-sizing: border-box;
}
@media screen and (max-width: 1200px) {
  .col:first-of-type {
    display: none;
  }
}

.side-row {
  width: 24.17%;
}
@media screen and (max-width: 1200px) {
  .side-row {
    width: 100%;
  }
}

.middle-row {
  width: 50%;
  margin: 20px 9px;
}
@media screen and (max-width: 1200px) {
  .middle-row {
    width: 100%;
    margin: 0 0 5px 0;
  }
}
.middle-row .articles p {
  line-height: 24px;
}
.middle-row .mid-con {
  padding: 0 29px;
}
@media screen and (max-width: 375px) {
  .middle-row .mid-con {
    padding: 0;
  }
}
.middle-row .mid-con h3 {
  margin-top: 35px;
  font-size: 20px;
  color: #1a76bc;
  margin-bottom: 22px;
  font-family: Dax;
}
.middle-row .mid-con p, form p {
  font-size: 15px;
  line-height: 24px;
  color: #59595c;
  margin: 16px 0;
}
.middle-row .mid-con ul li {
  font-size: 15px;
  line-height: 24px;
  color: #59595c;
}
.middle-row .mid-con a {
  color: #1a76bc;
  -webkit-transition: all 0.3s ease-in-out;
  -moz-transition: all 0.3s ease-in-out;
  -ms-transition: all 0.3s ease-in-out;
  -o-transition: all 0.3s ease-in-out;
  transition: all 0.3s ease-in-out;
}
.middle-row .mid-con a:hover {
  text-decoration: underline;
}

.top-block {
  background-color: #f4f5f7;
  min-height: 235px;
  padding: 45px 30px 30px 30px;
  box-sizing: border-box;
}
@media screen and (max-width: 1200px) {
  .top-block {
    min-height: inherit;
    padding: 45px 30px 30px 30px;
  }
}
.top-block h2 {
  font-family: Dax;
  font-size: 30px;
  color: #1a76bc;
  margin-bottom: 15px;
}
@media screen and (max-width: 375px) {
  .top-block h2 {
    text-align: left;
  }
}
.top-block p {
  margin-top: 15px;
  font-size: 15px;
  color: #59595c;
  line-height: 24px;
  padding: 0;
}

.round-block {
  background-repeat: no-repeat;
  background-position: center;
  background-size: cover;
  width: 290px;
  height: 235px;
  display: table;
  margin-bottom: 10px;
}
@media screen and (max-width: 1200px) {
  .round-block {
    display: inline-block;
    width: 31.5%;
    margin-left: -3px;
    margin-bottom: 15px;
    margin-right: 15px;
  }
}
@media screen and (max-width: 845px) {
  .round-block {
    width: 47%;
  }
}
@media screen and (max-width: 640px) {
  .round-block {
    width: 100%;
    margin-left: 0;
	background-position-y: -20px;
  }
}
@media screen and (max-width: 440px) {
  .round-block {
    width: 100%;
    margin-left: 0;
	background-position-y: -10px;
  }
}
.round-block span {
  font-family: Dax;
  color: white;
  text-transform: uppercase;
  display: table-cell;
}
.round-block span.middle {
  vertical-align: middle;
  padding-left: 34px;
  font-size: 30px;
}
.round-block span.top-text {
  padding: 21px 21px;
  font-size: 18px;
}

.side-menu {
  margin-top: 25px;
}
.side-menu ul li a {
  padding: 12px 32px;
  display: block;
  font-size: 15px;
  color: #1a76bc;
  -webkit-transition: all 0.3s ease-in-out;
  -moz-transition: all 0.3s ease-in-out;
  -ms-transition: all 0.3s ease-in-out;
  -o-transition: all 0.3s ease-in-out;
  transition: all 0.3s ease-in-out;
}
.side-menu ul li a.active, .side-menu ul li a:hover {
  color: white;
  font-weight: 700;
  background-color: #1a76bc;
}

.contact {
  min-height: 236px;
  width: 100%;
  background-repeat: no-repeat;
  background-position: center left;
  padding-left: 215px;
  box-sizing: border-box;
  margin: 10px 0;
  padding-top: 30px;
  background-color: #f4f5f7;
}
@media screen and (max-width: 1200px) {
  .contact {
    width: 100%;
    margin: 15px 0;
  }
}
@media screen and (max-width: 500px) {
  .contact {
    width: 300px;
    padding-top: 250px;
    padding-left: 0;
    background-position: top 15px center;
    padding-bottom: 15px;
    display: table;
    margin: 15px auto;
    background-size: auto 230px;
  }
}
.contact h3 {
  font-weight: 700;
  margin-top: 0;
  font-size: 24px;
  margin-bottom: 0px;
  color: #1a76bc;
}
@media screen and (max-width: 500px) {
  .contact h3 {
    margin: 10px auto;
    text-align: center;
  }
}
.contact span {
  font-size: 15px;
  display: block;
}
@media screen and (max-width: 500px) {
  .contact span {
    display: table;
    margin: 0 auto;
  }
}
.contact .amats {
  color: #59595c;
}
.contact .location {
  color: #59595c;
  margin-top: 31px;
}
@media screen and (max-width: 500px) {
  .contact .location {
    margin-top: 15px;
  }
}
.contact .mail {
  color: #1a76bc;
  padding-left: 29px;
  background-image: url(img/mail.png);
  background-repeat: no-repeat;
  background-position: left center;
  margin-top: 42px;
}
@media screen and (max-width: 500px) {
  .contact .mail {
    margin-top: 12px;
  }
}
.contact .tel {
  padding-left: 29px;
  background-image: url(img/tel.png);
  background-repeat: no-repeat;
  background-position: left center;
  color: #797779;
  margin-top: 12px;
}

.bjqs.no-resp {
display: block;
}
@media screen and (max-width: 1220px) {
.bjqs.no-resp {
display: none !important;
}
}
	.break {
		display: none;
	}
	.articles:nth-of-type(2n + 2) {
		margin-left: auto !Important;
	}
	.articles:nth-of-type(2n + 1) {
	}
	@media screen and (max-width: 768px) {
		.break {
		display: block;
		}
		.last {
			display: none;
		}
	}

	
footer .logo-aesc {
  position:absolute;
  right:0px;
  bottom:25px;
}
@media screen and (max-width: 1200px) {
  footer .logo-aesc {
  position:absolute;
  right:20px;
  top: 54px;
}
}
@media screen and (max-width: 768px) {
  footer .logo-aesc {
  position:absolute;
  right:20px;
  top: 72px;
  width:100px;
  height:auto;
}
}
@media screen and (max-width: 375px) {
  footer .logo-aesc {
  position:absolute;
  right:20px;
  top: 72px;
  width:100px;
  height:auto;
}
}

.mid-con ul li, .mid-con ol li {
list-style-type: square;
margin-left:50px;
margin-bottom:15px;
}

.mid-con ol li {
list-style-type: lower-alpha;
}

.widerow {
width: 900px;
margin-right: 0px;
}

@media screen and (max-width: 1200px) {
	.widerow {
		width: 100%;
	}
	.widerow .newsthumbs {
		text-align: center;
	}
	.widerow .newsthumbs li {
		width: calc(25% - 12px) !Important;
		float: none !Important;
		display: inline-block;
		margin-left: -4px;
	}
	.widerow .newsthumbs li img {
		width: 100% !important;
	}
	.widerow .newsthumbs li:nth-child(3n+3) {
		margin-right: 12px !important;
	}
	.widerow .newsthumbs li:nth-child(4n+4) {
		margin-right: 0px !Important;
	}
}
@media screen and (max-width: 950px) {
	.widerow .newsthumbs li {
		width: calc(33.33% - 12px) !Important;
		float: none !Important;
		display: inline-block;
		margin-left: -4px;
		margin-right: 17px !Important;
	}
	.widerow .newsthumbs li:nth-child(4n+4) {
		margin-right: 17px !Important;
	}
	.widerow .newsthumbs li:nth-child(3n+3) {
		margin-right: 0px !important;
	}
}
@media screen and (max-width: 690px) {
	.widerow .newsthumbs li {
		width: calc(50% - 12px) !Important;
		float: none !Important;
		display: inline-block;
		margin-left: -4px;
		margin-right: 17px !Important;
	}
	.widerow .newsthumbs li:nth-child(3n+3) {
		margin-right: 17px !important;
	}
	.widerow .newsthumbs li:nth-child(4n+4) {
		margin-right: 0px !Important;
	}
	.widerow .newsthumbs li:nth-child(2n+2) {
		margin-right: 0px !important;
	}
}
@media screen and (max-width: 500px) {
	.widerow .newsthumbs li {
		width: 100% !Important;
		float: none !Important;
		display: inline-block;
		margin-left: -4px;
		margin-right: 0px !Important;
	}
	.widerow .newsthumbs li:nth-child(3n+3) {
		margin-right: 0px !important;
	}
	.widerow .newsthumbs li:nth-child(4n+4) {
		margin-right: 0px !Important;
	}
	.widerow .newsthumbs li:nth-child(2n+2) {
		margin-right: 0px !important;
	}
}
.widerow .newsthumbs {
list-style:none;
padding-left:6px
}

.widerow .newsthumbs li {
float:left;
width:290px;
margin-right:12px;
padding-bottom:12px;
}

.widerow .newsthumbs li:nth-child(3n+3) {  
margin-right:0px;
}

.widerow .newsthumbs li img {
width:290px;
height:auto;
}

.newsthumbs2 {
list-style:none;
margin-right: 0px;
margin-left: 18px !important;
}

.newsthumbs2 li {
float:left;
width:289px;
margin-right:0px;
padding-bottom:15px;
}
@media screen and (max-width: 1200px) {
	.newsthumbs2 li {
		width: calc(50% - 5px);
	}
	.newsthumbs2 li a img {
		width: 100%;
	}
}
@media screen and (max-width: 1200px) {
	.newsthumbs2 li {
		width: 100%;
	}
	.newsthumbs2 li:nth-child(2n+1){
		margin-left: 0px !Important;
	}
}

@media screen and (max-width: 768px) {
	a + .newsthumbs2 .bttn1{
		float: none;
		margin-left: auto;
		margin-right: auto;
		margin-bottom: 15px;
	}
}

.newsthumbs2 li img {
width:289px;
height:auto;
}

.newsthumbs2 li:nth-child(2n+1) {  
margin-left:10px;
}