<?php
/**
* PHPMailer language file: refer to English translation for definitive list
* Traditional Chinese Version
* @author liqwei <liqwei@liqwei.com>
* @author Peter Dave Hello <https://github.com/PeterDaveHello/>
*/

$PHPMAILER_LANG['authenticate'] = 'SMTP 錯誤：登入失敗。';
$PHPMAILER_LANG['connect_host'] = 'SMTP 錯誤：無法連線到 SMTP 主機。';
$PHPMAILER_LANG['data_not_accepted'] = 'SMTP 錯誤：無法接受的資料。';
$PHPMAILER_LANG['empty_message'] = '郵件內容為空';
$PHPMAILER_LANG['encoding'] = '未知編碼: ';
$PHPMAILER_LANG['file_access'] = '無法存取檔案：';
$PHPMAILER_LANG['file_open'] = '檔案錯誤：無法開啟檔案：';
$PHPMAILER_LANG['from_failed'] = '發送地址錯誤：';
$PHPMAILER_LANG['execute'] = '無法執行：';
$PHPMAILER_LANG['instantiate'] = '未知函數呼叫。';
$PHPMAILER_LANG['invalid_address'] = '因為電子郵件地址無效，無法傳送: ';
$PHPMAILER_LANG['provide_address'] = '必須提供至少一個收件人地址。';
$PHPMAILER_LANG['mailer_not_supported'] = '不支援的發信客戶端。';
$PHPMAILER_LANG['recipients_failed'] = 'SMTP 錯誤：收件人地址錯誤：';
$PHPMAILER_LANG['signing']              = '登入失敗: ';
$PHPMAILER_LANG['smtp_connect_failed']  = 'SMTP連線失敗';
$PHPMAILER_LANG['smtp_error']           = 'SMTP伺服器錯誤: ';
$PHPMAILER_LANG['variable_set']         = '無法設定或重設變數: ';
