.form-table.aioseop th {
    padding: 10px 9px 12px 0;
}
.aioseop_help_text_link, .aioseop_help_text_link:active {
    text-align: right;
    float: right;
    background: url('images/question.png') no-repeat top right;
}
.aioseop_help_text_link img {
    float: right;
}
.aioseop_label {
    padding-right: 0px;
    padding-left: 10px;
    text-align: right;
}
.aioseop input[type="text"] {
    padding: 10px 10px 10px 0px;
}
.aioseop textarea {
    padding: 10px 10px 0px 0px;
}
.aioseop_help_text_div {
    text-align:right;
    margin: 10px 0px 10px 0px;
}
.aioseop_help_text {
    float: right;
    clear: right;
    font-style: normal;
}

.aioseop_head_nav {
    float: right;
}
.aioseop_head_nav_tab {
    margin: 0 15px 0 0;
    border-width: 1px 1px 0;
    border-radius: 4px 0 0px 4px;
    float: right;
}

.aioseop_head_nav_tab:first-child {
    margin-right: 0px;
    margin-left: auto;
}
.aioseop_header {
    float: right;
    clear: right;
}

.aioseop_adverts {
    float: left;
}
.aioseop_content {
    clear: right;
}
#aiosp_feature_manager_metabox.postbox {
    float: right;
}
.aioseop_sidebar {
    width: 457px;
    margin-right: 10px;
    margin-left: auto;
}
.aioseop_option_label {
    float: right;
    margin: 0px 5px 10px 0;
}
.aioseop_settings_left {
    float: right;
}

.aioseop_right_sidebar {
    float: left;
}

.aioseop_input {
    clear: right;
}
.aioseop_option_input {
    float: right;
    width: 61%;
    padding-right: 1px;
    padding-left: 0px;
}
#aiosp_sitemap_addl_pages {
    clear: right;
    margin-right: 20px;
    margin-left: auto;
    max-width: 1072px;
}
#aiosp_sitemap_addl_pages_metabox table.aioseop_table td {
    padding: 3px;
    padding-right: 5%;
}
.aioseop_top_label .aioseop_option_input,
.aioseop_no_label .aioseop_option_input {
    width: 100%;
}

.aioseop_settings_left .postbox {
    float: right;
}
.aioseop_option_setting_label {
    padding-right: 1px;
    padding-left: 0px;
}
#aiosp_settings_form .aioseop_no_label, .aioseop_no_label {
    float: right;
    margin: 0px 13px 0px 23px;
}
.aioseop_module.error.below-h2 {
    padding: 5px 0px;
    margin: 0px 0px 15px 477px !important;
}

#aiosp {
    width: 100%;
}

.aioseop_help_text ul {
    margin: 15px 20px 0 0;
}

.aioseop_header_tabs li a.aioseop_header_tab {
    border: solid 1px #CCC;
    border-right: none;
    padding: 6px 6px 1px 6px;
}

form#aiosp_settings_form, .aioseop_tabs_div {
    padding-left: 477px;
    padding-right: 0px;
}
#aiosp_settings_form ul.sfwd_debug_settings li strong {
    float: right;
    text-align: left;
    margin-left: 8px;
    margin-right: auto;
    padding: 1px;
    padding-left: 8px;
}