$(document).ready(function(){
    var w = $(window).width(),
        s = $(".slider li"),
        b = $(".bjqs"),
        l = s.length,
        a = $(".drop"),
        c = $(".ham");

    // s.width(w+"px");
    //b.width(w*l+"px");

    a.click(function(){$(this).siblings().find("ul").slideUp();$(this).children("ul").slideToggle()});
    //a.removeAttr("href");
    c.click(function(){$(".resp-drop").slideToggle()});
});