<?php
/*
Template Name: Services
*/

$page = get_page(get_the_ID());
$currentlang = pll_current_language();?>
<?php get_header(); ?>
<div class="con">
	<div class="col side-row">
		<div class="round-block" style="background-image:url(img/services.png)">
			<span class="middle"><?=get_the_title(($currentlang=='eng'?5:74))?></span>
		</div>
		<nav class="side-menu">
			<ul>
				<?php $args = array(
						'sort_order' => 'asc',
						'sort_column' => 'menu_order',
						'hierarchical' => 1,
						'parent' => ($currentlang=='eng'?5:74),
						'post_type' => 'page',
						'post_status' => 'publish'
					); 
					$pgs = get_pages($args); 
					foreach ( $pgs as $pg ) {
						echo '<li><a '.($page->ID == $pg->ID?'class="active"':'').' href="'. get_page_link( $pg->ID ) .'">'. $pg->post_title .'</a></li>';
					}
				?>
			</ul>
		</nav>
	</div>

	<div class="col middle-row">
		<div class="top-block">
			<h2><?=$page->post_title?></h2>
			<?=apply_filters('the_content', $page->post_content)?>
		</div>
		<div class="mid-con">
			<?the_field("lower_text");?>
		</div>
	</div>
	<?get_sidebar();?>
</div>
<?php get_footer(); ?>