<?php
/**
* PHPMailer language file: refer to English translation for definitive list
* Spanish version
* Versión en español
* Edited by Matt Sturdy - matt.sturdy@gmail.com
*/

$PHPMAILER_LANG['authenticate']         = 'Error SMTP: No se pudo autentificar.';
$PHPMAILER_LANG['connect_host']         = 'Error SMTP: No se pudo conectar al servidor SMTP.';
$PHPMAILER_LANG['data_not_accepted']    = 'Error SMTP: Datos no aceptados.';
$PHPMAILER_LANG['empty_message']        = 'Cuerpo del mensaje vacío';
$PHPMAILER_LANG['encoding']             = 'Codificación desconocida: ';
$PHPMAILER_LANG['execute']              = 'No se pudo ejecutar: ';
$PHPMAILER_LANG['file_access']          = 'No se pudo acceder al archivo: ';
$PHPMAILER_LANG['file_open']            = 'Error de Archivo: No se pudo abrir el archivo: ';
$PHPMAILER_LANG['from_failed']          = 'La(s) siguiente(s) direcciones de remitente fallaron: ';
$PHPMAILER_LANG['instantiate']          = 'No se pudo crear una instancia de la función Mail.';
$PHPMAILER_LANG['invalid_address']      = 'No se pudo enviar: dirección de email inválido: ';
$PHPMAILER_LANG['mailer_not_supported'] = ' mailer no está soportado.';
$PHPMAILER_LANG['provide_address']      = 'Debe proveer al menos una dirección de email como destino.';
$PHPMAILER_LANG['recipients_failed']    = 'Error SMTP: Los siguientes destinos fallaron: ';
$PHPMAILER_LANG['signing']              = 'Error al firmar: ';
$PHPMAILER_LANG['smtp_connect_failed']  = 'SMTP Connect() se falló.';
$PHPMAILER_LANG['smtp_error']           = 'Error del servidor SMTP: ';
$PHPMAILER_LANG['variable_set']         = 'No se pudo ajustar o reajustar la variable: ';
