<?php
/**
* PHPMailer language file: refer to English translation for definitive list
* Norwegian Version
*/

$PHPMAILER_LANG['authenticate']         = 'SMTP Feil: Kunne ikke authentisere.';
$PHPMAILER_LANG['connect_host']         = 'SMTP Feil: Kunne ikke koble til SMTP host.';
$PHPMAILER_LANG['data_not_accepted']    = 'SMTP Feil: Data ble ikke akseptert.';
$PHPMAILER_LANG['empty_message']        = 'Meldingsinnholdet er tomt';
$PHPMAILER_LANG['encoding']             = 'Ukjent tegnkoding: ';
$PHPMAILER_LANG['execute']              = 'Kunne ikke utføre: ';
$PHPMAILER_LANG['file_access']          = 'Får ikke tilgang til filen: ';
$PHPMAILER_LANG['file_open']            = 'Fil feil: Kunne ikke åpne filen: ';
$PHPMAILER_LANG['from_failed']          = 'Følgende avsenderadresse feilet: ';
$PHPMAILER_LANG['instantiate']          = 'Kunne ikke initialisere mailfunksjonen.';
$PHPMAILER_LANG['invalid_address']      = 'Meldingen ble ikke sendt, følgende adresse er ugyldig: ';
$PHPMAILER_LANG['provide_address']      = 'Du må angi minst en mottakeradresse.';
$PHPMAILER_LANG['mailer_not_supported'] = ' mailer er ikke supportert.';
$PHPMAILER_LANG['recipients_failed']    = 'SMTP Feil: Følgende mottagere feilet: ';
$PHPMAILER_LANG['signing']              = 'Signeringsfeil: ';
$PHPMAILER_LANG['smtp_connect_failed']  = 'SMTP Connect() feilet.';
$PHPMAILER_LANG['smtp_error']           = 'SMTP-serverfeil: ';
$PHPMAILER_LANG['variable_set']         = 'Kan ikke sette eller resette variabelen: ';
