<?
$sideImage1 = get_field('sidebar_image_1');
$sideImage2 = get_field('sidebar_image_2');
if(!$sideImage1 && $post->post_parent != 0){
	$sideImage1 = get_field('sidebar_image_1', $post->post_parent);
}
if(!$sideImage2 && $post->post_parent != 0){
	$sideImage2 = get_field('sidebar_image_2', $post->post_parent);
}
$currentlang = pll_current_language();
?>
<div class="col side-row">
	<?php
	global $post;
	$pageid = $post->ID;
	?>
	<? if($pageid == ('25')){?>
	<div class="round-block" style="background-image:url('img/executive-search-graph.png'); height:552px;"></div>
	<? } elseif($pageid == ('78')){?>
	<div class="round-block" style="background-image:url('img/executive-search-graph-est.png'); height:552px;"></div>
	<? } elseif($pageid == ('27')){?>
	<div class="round-block" style="background-image:url('img/leadership_eng.jpg'); height:1026px;"></div>
	<? } elseif($pageid == ('77')){?>
	<div class="round-block" style="background-image:url('img/leadership_est.jpg'); height:1026px;"></div>
	<? } elseif($pageid == ('376')){?>
	<div class="round-block" style="background-image:url('img/board_eng.png'); height:442px;"></div>
	<? } elseif($pageid == ('378')){?>
	<div class="round-block" style="background-image:url('img/board_est.png'); height:442px;"></div>
	<? } elseif($pageid == ('102')){?>
	<div class="round-block" style="background-image:url('img/5dimensions_eng.png'); height:442px;"></div>
	<? } elseif($pageid == ('104')){?>
	<div class="round-block" style="background-image:url('img/5dimensions_est.png'); height:442px;"></div>
	<?}else{?>
	<?if($sideImage1){?><div class="round-block" style="background-image:url('<?=$sideImage1['sizes']['sidebar_image_img']?>')"></div><?}?>
	<?if($sideImage2){?><div class="round-block" style="background-image:url('<?=$sideImage2['sizes']['sidebar_image_img']?>')"></div><?}?>
	<? } ?>
	<a href="http://amrop.com/offices" target="_blank" class="round-block" style="background-image:url(img/services-4.png)">
		<span class="top-text">
			<?=($currentlang=='eng'?'Amrop global offices':'AMROP ÜLE MAAILMA')?>
		</span>
	</a>
</div>