<?php
/**
* PHPMailer language file: refer to English translation for definitive list
* Romanian Version
* @package PHPMailer
* @author Catalin Constantin <catalin@dazoot.ro>
*/

$PHPMAILER_LANG['authenticate']         = 'Eroare SMTP: Nu a functionat autentificarea.';
$PHPMAILER_LANG['connect_host']         = 'Eroare SMTP: Nu m-am putut conecta la adresa SMTP.';
$PHPMAILER_LANG['data_not_accepted']    = 'Eroare SMTP: Continutul mailului nu a fost acceptat.';
$PHPMAILER_LANG['empty_message']        = 'Mesajul este gol.';
$PHPMAILER_LANG['encoding']             = 'Encodare necunoscuta: ';
$PHPMAILER_LANG['execute']              = 'Nu pot executa:  ';
$PHPMAILER_LANG['file_access']          = 'Nu pot accesa fisierul: ';
$PHPMAILER_LANG['file_open']            = 'Eroare de fisier: Nu pot deschide fisierul: ';
$PHPMAILER_LANG['from_failed']          = 'Urmatoarele adrese From au dat eroare: ';
$PHPMAILER_LANG['instantiate']          = 'Nu am putut instantia functia mail.';
$PHPMAILER_LANG['invalid_address']      = 'Adresa de email nu este valida. ';
$PHPMAILER_LANG['mailer_not_supported'] = ' mailer nu este suportat.';
$PHPMAILER_LANG['provide_address']      = 'Trebuie sa adaugati cel putin un recipient (adresa de mail).';
$PHPMAILER_LANG['recipients_failed']    = 'Eroare SMTP: Urmatoarele adrese de mail au dat eroare: ';
$PHPMAILER_LANG['signing']              = 'A aparut o problema la semnarea emailului. ';
$PHPMAILER_LANG['smtp_connect_failed']  = 'Conectarea la serverul SMTP a esuat.';
$PHPMAILER_LANG['smtp_error']           = 'A aparut o eroare la serverul SMTP. ';
$PHPMAILER_LANG['variable_set']         = 'Nu se poate seta/reseta variabila. ';
