<?php
/**
* PHPMailer language file: refer to English translation for definitive list
* Georgian Version by Avtandil Kikabidze aka LONGMAN <akalongman@gmail.com>
*/

$PHPMAILER_LANG['authenticate'] 			= 'SMTP შეცდომა: ავტორიზაცია შეუძლებელია.';
$PHPMAILER_LANG['connect_host']  			= 'SMTP შეცდომა: SMTP სერვერთან დაკავშირება შეუძლებელია.';
$PHPMAILER_LANG['data_not_accepted'] 		= 'SMTP შეცდომა: მონაცემები არ იქნა მიღებული.';
$PHPMAILER_LANG['encoding'] 				= 'კოდირების უცნობი ტიპი: ';
$PHPMAILER_LANG['execute'] 					= 'შეუძლებელია შემდეგი ბრძანების შესრულება: ';
$PHPMAILER_LANG['file_access'] 				= 'შეუძლებელია წვდომა ფაილთან: ';
$PHPMAILER_LANG['file_open'] 				= 'ფაილური სისტემის შეცდომა: არ იხსნება ფაილი: ';
$PHPMAILER_LANG['from_failed'] 				= 'გამგზავნის არასწორი მისამართი: ';
$PHPMAILER_LANG['instantiate'] 				= 'mail ფუნქციის გაშვება ვერ ხერხდება.';
$PHPMAILER_LANG['provide_address'] 		= 'გთხოვთ მიუთითოთ ერთი ადრესატის e-mail მისამართი მაინც.';
$PHPMAILER_LANG['mailer_not_supported'] 	= ' - საფოსტო სერვერის მხარდაჭერა არ არის.';
$PHPMAILER_LANG['recipients_failed'] 		= 'SMTP შეცდომა: შემდეგ მისამართებზე გაგზავნა ვერ მოხერხდა: ';
$PHPMAILER_LANG['empty_message'] 		= 'შეტყობინება ცარიელია';
$PHPMAILER_LANG['invalid_address'] 			= 'არ გაიგზავნა, e-mail მისამართის არასწორი ფორმატი: ';
$PHPMAILER_LANG['signing'] 					= 'ხელმოწერის შეცდომა: ';
$PHPMAILER_LANG['smtp_connect_failed'] 	= 'შეცდომა SMTP სერვერთან დაკავშირებისას';
$PHPMAILER_LANG['smtp_error'] 				= 'SMTP სერვერის შეცდომა: ';
$PHPMAILER_LANG['variable_set'] 				= 'შეუძლებელია შემდეგი ცვლადის შექმნა ან შეცვლა: ';
