<footer>
		<div class="con">
			<img src="img/logo-sm.png" class="logo-sm"/>
			<ul>
				<h4>Amrop</h4>
				<li><a href="http://amrop.ee">Estonia</a></li>
				<li><a href="http://amrop.lv">Latvia</a></li>
				<li><a href="http://amrop.lt">Lithuania</a></li>
				<li><a href="http://amrop.com">Global</a></li>
			</ul>
			<?	$args = array(
					'sort_order' => 'asc',
					'sort_column' => 'menu_order',
					'hierarchical' => 1,
					'parent' => '',
					'post_type' => 'page',
					'post_status' => 'publish',
					'exclude' => array(46,53,401,403,745,753)
				); 
				$pgs = get_pages($args); 
				foreach ( $pgs as $pg ) {
					echo '<ul>';
					echo '<h4><a href="'. get_page_link( $pg->ID ) .'">'. $pg->post_title .'</a></h4>';
					if($pg->ID != 9){
						if($pg->ID == 14){
							echo '<li><a href="'. get_page_link( $pg->ID ) .'#SendCV">Send CV</a></li>';
						}else{
							$subArgs = array(
								'sort_order' => 'desc',
								'sort_column' => 'menu_order',
								'hierarchical' => 1,
								'parent' => $pg->ID,
								'post_type' => 'page',
								'post_status' => 'publish'
							); 
							$subPgs = get_pages($subArgs);
							foreach ( $subPgs as $sPg ) {
								echo '<li><a href="'. get_page_link( $sPg->ID ) .'">'. $sPg->post_title .'</a></li>';
							}
						}
					}
					echo '</ul>';
				}
			?>
			<span class="f-left"><?=($currentlang=='eng'?'© 2015 Amrop. All rights reserved. <a style="color:#b4b5b6; text-decoration:underline;" href="http://www.amrop.ee/eng/privacy-policy">Privacy policy.</a>':'© 2015 Amrop. Kõik õigused kaitstud. <a style="color:#b4b5b6; text-decoration:underline;" href="http://www.amrop.ee/est/privaatsuspoliitika">Privaatsuspoliitika.</a>')?> </span>
		</div>
	</footer>
</div>
<script src="js/main.js"></script>