<?php
function register_my_menu() {
  register_nav_menu('header-menu', 'Main Menu' );
}
add_action( 'init', 'register_my_menu' );
add_theme_support( 'post-thumbnails', array( 'post', 'page' ) );
add_image_size( 'teksta_lapa_top_img', 738, 421, true);
add_image_size( 'sidebar_image_img', 293, 236, true);
add_image_size( 'team_member', 185, 237, true);
add_filter('nav_menu_css_class' , 'special_nav_class' , 10 , 2);
function special_nav_class($classes, $item){
	$tempClasses = $classes;
	if(in_array('menu-item-has-children', $tempClasses)){
		$classes = array('drop');
	}else{
		$classes = array();
	}
    return $classes;
}
?>