<li class="news-item" >
	<p>
		<?php 
		if(function_exists('relevanssi_the_title')){
			echo '<a href="'.esc_url( get_permalink() ).'"><h3>';
			relevanssi_the_title(); 
			echo '</h3></a>';
		}else{
			the_title( sprintf( '<a href="%s"><h3>', esc_url( get_permalink() ) ), '</h3></a>' );
		}?><br>
		<? relevanssi_the_excerpt() ?>
	</p>
</li>