/* languages admin panel */
.column-term_group, .column-flag, .column-count {
	width : 10%;
}

.form-field input[type="radio"] {
	width: auto;
	margin-right: 2px;
}

/* about Polylang metabox */
#pll_about_box p, #pll_recommended p {
	text-align: justify;
}

#pll_about_box input {
	margin: 0;
	padding: 0;
	float: right;
}

/* strings translation table */
#string-translation .search-box {
	margin: 16px 0 8px 0;
}

.stringstranslations .column-name, .stringstranslations .column-context {
	width: 10%;
}

.stringstranslations .column-string {
	width: 33%;
}

.translation label {
	display: inline-block;
	width: 23%;
	vertical-align: top;
}

.translation input, .translation textarea {
	width: 72%;
}

/* settings */
.pll_inline_block {
	margin: 0;
}

.pll_inline_block li {
	display: inline-block;
	margin: 0;
	width: 250px;
}

#pll-domains-table td {
	padding: 2px 2px 2px 1.5em;
}

#pll-force-lang .description {
	margin-top: 0;
	margin-bottom: 0.5em;
}

/* language columns in edit.php and edit-tags.php */
th[class*='column-language_'], td[class*='column-language_'] {
	width: 2em;
}


/* languages metabox in post.php */
#post-translations table {
	table-layout: fixed;
	width: 100%;
}

#post-translations a {
	text-decoration: none;
}

#post-translations .pll-language-column, #post-translations .pll-edit-column {
	width: 20px;
}

#post-translations td {
	padding: 2px;
}

#post-translations .spinner, #term-translations .spinner {
	float: none;
	margin: 0;
	background-position: center;
	width: auto;
}

.pll-edit-column {
	text-align: center;
}

#select-post-language .pll-select-flag {
	padding: 4px;
	margin-right: 32px;
}

/* specific cases for media */
#select-media-language .pll-select-flag {
	padding: 4px;
	margin-right: 10px;
}

.pll-media-edit-column {
	float: right;
}

/* language and translations in edit-tags.php */
.pll-translation-flag { /* also for media */
	margin-right: 14px;
}

#select-add-term-language .pll-select-flag {
	padding: 11px;
	margin-right: 13px;
}

#select-edit-term-language .pll-select-flag {
	padding: 11px;
	margin-right: 4px;
}

#add-term-translations, #edit-term-translations {
	width: 95%;
}

#term-translations .pll-language-column {
  line-height: 28px;
  width: 20%;
}

#term-translations .pll-edit-column, #add-term-translations .pll-language-column {
  width: 20px;
}

/* icon fonts */
.pll_icon_tick:before {
	content: "\f147";
}
.pll_icon_add:before {
	content: "\f132";
}
.pll_icon_edit:before {
	content: "\f464";
}

[class^="pll_icon_"] {
	font: 20px/1 'dashicons';
	vertical-align: middle;
}

/* admin bar */
#wpadminbar #wp-admin-bar-languages .ab-item img {
	margin: 0 8px 0 2px;
}

#wpadminbar #wp-admin-bar-languages #wp-admin-bar-all .ab-item .ab-icon {
	float: none;
	top: 4px;
}

#wpadminbar #wp-admin-bar-languages .ab-icon:before {
	content: "\f326";
	top: 1px;
}

@media screen and ( max-width: 782px ) {
	/* strings translations table */
	.stringstranslations .column-context,
	.stringstranslations .column-name {
		display: none;
	}

	/* hide selected language flag and translations language name */
	#select-add-term-language .pll-select-flag,
	#select-edit-term-language .pll-select-flag,
	#edit-term-translations .pll-language-name {
		display: none;
	}

	#edit-term-translations {
		width: 100%;
	}

	#add-term-translations .pll-language-column {
		line-height: 38px;
	}

	#edit-term-translations td {
		padding: 8px 10px;
	}

	#edit-term-translations .pll-language-column,
	#edit-term-translations .pll-edit-column {
		width: 20px;
	}

	/* translations tables should be kept as table */
	.term-translations .pll-language-column,
	.term-translations .pll-edit-column,
	.term-translations .pll-translation-column {
		display: table-cell;
	}

	.term-translations .hidden {
		display: none;
	}

	/* admin bar */
	#wpadminbar li#wp-admin-bar-languages {
		display: block; /*shows our menu on mobile devices */
	}

	#wpadminbar #wp-admin-bar-languages > .ab-item {
		width: 50px;
		text-align: center;
	}

	#wpadminbar #wp-admin-bar-languages > .ab-item .ab-icon:before {
		font: 32px/1 'dashicons';
		top: -1px;
	}

	#wpadminbar #wp-admin-bar-languages > .ab-item img {
		margin: 19px 0;
	}

	#wpadminbar #wp-admin-bar-languages #wp-admin-bar-all .ab-item .ab-icon {
		margin-right: 6px;
		font-size: 20px !important;
		line-height: 20px !important;
	}
}
