<?php

$page = get_page(get_the_ID());
$currentlang = pll_current_language();
get_header();?>
<div class="con">
	<div class="col side-row">
		<div class="round-block" style="background-image:url(img/services.png)">
			<span class="middle"><?=($currentlang=='eng'?'Search':'Otsi')?></span>
		</div>
	</div>
	<div class="col middle-row">
		<div class="top-block">
			<h2><?=($currentlang=='eng'?'Results for':'Otsingutulemused')?></h2>
			<?=$_REQUEST['s']?>
		</div>
		<ul class="articles">
			<?
			while ( have_posts() ) : the_post();
				get_template_part( 'content', 'search' );
			endwhile;
			?>
		</ul>
	</div>

	<div class="col side-row">
		<div class="round-block" style="background-image:url(img/services-2.png)"></div>
		<div class="round-block" style="background-image:url(img/services-3.png)"></div>
		<a href="http://amrop.com/offices" target="_blank" class="round-block" style="background-image:url(img/services-4.png)">
			<span class="top-text">
				<?=($currentlang=='eng'?'Amrop global offices':'AMROP ÜLE MAAILMA')?>
			</span>
		</a>
	</div>
</div>
<?php get_footer(); ?>