<?php
/*
 * for each language:
 * [0] => ISO 639-1 language code
 * [1] => WordPress locale
 * [2] => name
 * [3] => not set for ltr scripts, 'rtl' otherwise
 */
$languages = array(
	array('af', 'af', 'Afrikaans'),
	array('ar', 'ar', 'العربية', 'rtl'),
	array('az', 'az', 'Azərbaycan'),
	array('be', 'bel', 'Беларуская мова'),
	array('bg', 'bg_BG', 'български'),
	array('bs', 'bs_BA', 'Bosanski'),
	array('ca', 'ca', 'Català'),
	array('cs', 'cs_CZ', 'Čeština'),
	array('cy', 'cy', 'Cymraeg'),
	array('da', 'da_DK', 'Dansk'),
	array('de', 'de_CH', 'Deutsch'),
	array('de', 'de_DE', 'Deutsch'),
	array('el', 'el', 'Ελληνικά'),
	array('en', 'en_AU', 'English'),
	array('en', 'en_CA', 'English'),
	array('en', 'en_GB', 'English'),
	array('en', 'en_US', 'English'),
	array('eo', 'eo', 'Esperanto'),
	array('es', 'es_CL', 'Español'),
	array('es', 'es_ES', 'Español'),
	array('es', 'es_MX', 'Español'),
	array('es', 'es_PE', 'Español'),
	array('es', 'es_VE', 'Español'),
	array('et', 'et', 'Eesti'),
	array('eu', 'eu', 'Euskara'),
	array('fa', 'fa_AF', 'فارسی', 'rtl'),
	array('fa', 'fa_IR', 'فارسی', 'rtl'),
	array('fi', 'fi', 'Suomi'),
	array('fo', 'fo', 'Føroyskt'),
	array('fr', 'fr_FR', 'Français'),
	array('fy', 'fy', 'Frysk'),
	array('gd', 'gd', 'Gàidhlig'),
	array('gl', 'gl_ES', 'Galego'),
	array('haz', 'haz', 'هزاره گی', 'rtl'),
	array('he', 'he_IL', 'עברית', 'rtl'),
	array('hi', 'hi_IN', 'हिन्दी'),
	array('hr', 'hr', 'Hrvatski'),
	array('hu', 'hu_HU', 'Magyar'),
	array('id', 'id_ID', 'Bahasa Indonesia'),
	array('is', 'is_IS', 'Íslenska'),
	array('it', 'it_IT', 'Italiano'),
	array('ja', 'ja', '日本語'),
	array('jv', 'jv_ID', 'Basa Jawa'),
	array('ka', 'ka_GE', 'ქართული'),
	array('kk', 'kk', 'Қазақ тілі'),
	array('ko', 'ko_KR', '한국어'),
	array('ku', 'ckb', 'کوردی', 'rtl'),
	array('lo', 'lo', 'ພາສາລາວ'),
	array('lt', 'lt_LT', 'Lietuviškai'),
	array('lv', 'lv', 'Latviešu valoda'),
	array('mk', 'mk_MK', 'македонски јазик'),
	array('mn', 'mn', 'Монгол хэл'),
	array('ms', 'ms_MY', 'Bahasa Melayu'),
	array('my', 'my_MM', 'ဗမာစာ'),
	array('nb', 'nb_NO', 'Norsk Bokmål'),
	array('ne', 'ne_NP', 'नेपाली'),
	array('nl', 'nl_NL', 'Nederlands'),
	array('nn', 'nn_NO', 'Norsk Nynorsk'),
	array('pl', 'pl_PL', 'Polski'),
	array('pt', 'pt_BR', 'Português'),
	array('pt', 'pt_PT', 'Português'),
	array('ro', 'ro_RO', 'Română'),
	array('ru', 'ru_RU', 'Русский'),
	array('si', 'si_LK', 'සිංහල'),
	array('sk', 'sk_SK', 'Slovenčina'),
	array('sl', 'sl_SI', 'Slovenščina'),
	array('so', 'so_SO', 'Af-Soomaali'),
	array('sq', 'sq', 'Shqip'),
	array('sr', 'sr_RS', 'Српски језик'),
	array('su', 'su_ID', 'Basa Sunda'),
	array('sv', 'sv_SE', 'Svenska'),
	array('ta', 'ta_LK', 'தமிழ்'),
	array('th', 'th', 'ไทย'),
	array('tr', 'tr_TR', 'Türkçe'),
	array('ug', 'ug_CN', 'Uyƣurqə'),
	array('uk', 'uk', 'Українська'),
	array('ur', 'ur', 'اردو', 'rtl'),
	array('uz', 'uz_UZ', 'Oʻzbek'),
	array('vec', 'vec', 'Vèneto'),
	array('vi', 'vi', 'Tiếng Việt'),
	array('zh', 'zh_CN', '中文 (中国)'),
	array('zh', 'zh_HK', '中文 (香港)'),
	array('zh', 'zh_TW', '中文 (台灣)'),
);
