<?php
/**
 * Zend Framework
 *
 * LICENSE
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://framework.zend.com/license/new-bsd
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@zend.com so we can send you a copy immediately.
 *
 * @category  Microsoft
 * @package   Microsoft_Uri
 * @copyright Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
 * @license   http://framework.zend.com/license/new-bsd     New BSD License
 * @version   $Id: Exception.php 16208 2009-06-21 19:19:26Z thomas $
 */
if (!defined('W3TC')) {
    die();
}

/**
 * @see Microsoft_Exception
 */
require_once 'Microsoft/Exception.php';

/**
 * Exceptions for Microsoft_Uri
 *
 * @category  Microsoft
 * @package   Microsoft_Uri
 * @copyright Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
 * @license   http://framework.zend.com/license/new-bsd     New BSD License
 */
class Microsoft_Uri_Exception extends Microsoft_Exception
{
}
