<?php

/**
 * HTML mime types
 */
return array(
    'html|htm' => 'text/html', 
    'rtf|rtx' => 'text/richtext', 
    'svg|svgz' => 'image/svg+xml', 
    'txt' => 'text/plain', 
    'xsd' => 'text/xsd', 
    'xsl' => 'text/xsl', 
    'xml' => 'text/xml'
);
