<?php
/*
Template Name: About
*/

$page = get_page(get_the_ID());
$currentlang = pll_current_language();?>
<?php get_header(); ?>
<div class="con">
	<div class="col side-row">
		<div class="round-block" style="background-image:url(img/aboutus.jpg)">
			<span class="middle"><?=$page->post_title?></span>
		</div>
		<nav class="side-menu">
			<ul>
				<li><a href="http://amrop.lv"><?=($currentlang=='eng'?'Latvia':'Läti')?></a></li>
				<li><a href="http://amrop.lt"><?=($currentlang=='eng'?'Lithuania':'Leedu')?></a></li>
				<li><a href="http://amrop.com"><?=($currentlang=='eng'?'Global':'Global')?></a></li>
			</ul>
		</nav>
	</div>

	<div class="col middle-row">
		<div class="top-block">
			<h2><?=$page->post_title?></h2>
			<?=apply_filters('the_content', $page->post_content)?>
		</div>
		<div class="mid-con">
			<?the_field("lower_text");?>
		</div>
	</div>
	<?get_sidebar();?>
</div>
<?php get_footer(); ?>